@@ -1,4 +1,4 @@
-/* Copyright (c) 2007 Advanced Micro Devices, Inc.
+/* Copyright (c) 2007-2008 Advanced Micro Devices, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -38,7 +38,6 @@
 
 #include "xf86.h"
 #include "xf86_OSproc.h"
-#include "xf86Resources.h"
 #include "compiler.h"
 #include "xf86PciInfo.h"
 #include "xf86Pci.h"
@@ -100,8 +99,7 @@ static XF86ImageRec Images[] = {
 
 typedef struct
 {
-    void *area;
-    int offset;
+    ExaOffscreenArea *vidmem;
     RegionRec clip;
     CARD32 filter;
     CARD32 colorKey;
@@ -121,7 +119,7 @@ LXCopyFromSys(GeodeRec * pGeode, unsigned char *src, unsigned int dst,
 {
 
     gp_declare_blt(0);
-    gp_set_bpp(16);
+    gp_set_bpp((srcPitch / w) << 3);
 
     gp_set_raster_operation(0xCC);
     gp_set_strides(dstPitch, srcPitch);
@@ -130,38 +128,6 @@ LXCopyFromSys(GeodeRec * pGeode, unsigned char *src, unsigned int dst,
     gp_color_bitmap_to_screen_blt(dst, 0, w, h, src, srcPitch);
 }
 
-static void
-LXVideoSave(ScreenPtr pScreen, ExaOffscreenArea * area)
-{
-    ScrnInfoPtr pScrni = xf86Screens[pScreen->myNum];
-
-    GeodePortPrivRec *pPriv = GET_PORT_PRIVATE(pScrni);
-
-    if (area == pPriv->area)
-	pPriv->area = NULL;
-
-    LXStopVideo(pScrni, (void *)pPriv, TRUE);
-}
-
-static unsigned int
-LXAllocateVidMem(ScrnInfoPtr pScrni, void **memp, int size)
-{
-    ExaOffscreenArea *area = *memp;
-
-    if (area != NULL) {
-	if (area->size >= size)
-	    return area->offset;
-
-	exaOffscreenFree(pScrni->pScreen, area);
-    }
-
-    area = exaOffscreenAlloc(pScrni->pScreen, size, 16, TRUE,
-	LXVideoSave, NULL);
-
-    *memp = area;
-    return (area == NULL) ? 0 : area->offset;
-}
-
 static void
 LXSetColorkey(ScrnInfoPtr pScrni, GeodePortPrivRec * pPriv)
 {
@@ -221,6 +187,27 @@ struct
 
 /* Copy planar YUV data */
 
+static Bool
+LXAllocateVidMem(ScrnInfoPtr pScrni, GeodePortPrivRec *pPriv, int size)
+{
+    if (!pPriv->vidmem || pPriv->vidmem->size < size) { 
+	if (pPriv->vidmem) {
+		exaOffscreenFree(pScrni->pScreen, pPriv->vidmem);
+		pPriv->vidmem = NULL;
+	}
+
+	pPriv->vidmem = exaOffscreenAlloc(pScrni->pScreen, size, 4,
+			TRUE, NULL, NULL);
+
+    	if (pPriv->vidmem == NULL) {
+		ErrorF("Could not allocate memory for the video\n");
+		return FALSE;
+    	}
+    }
+
+    return TRUE;
+}	
+
 static Bool
 LXCopyPlanar(ScrnInfoPtr pScrni, int id, unsigned char *buf,
     short x1, short y1, short x2, short y2,
@@ -252,10 +239,8 @@ LXCopyPlanar(ScrnInfoPtr pScrni, int id, unsigned char *buf,
     size = YDstPitch * height;
     size += UVDstPitch * height;
 
-    pPriv->offset = LXAllocateVidMem(pScrni, &pPriv->area, size);
-
-    if (pPriv->offset == 0) {
-	ErrorF("Error allocating an offscreen region.\n");
+    if (LXAllocateVidMem(pScrni, pPriv, size) == FALSE) {
+	ErrorF("Error allocating an offscreen Planar region.\n");
 	return FALSE;
     }
 
@@ -282,19 +267,21 @@ LXCopyPlanar(ScrnInfoPtr pScrni, int id, unsigned char *buf,
 
     /* Copy Y */
 
-    LXCopyFromSys(pGeode, buf + YSrcOffset, pPriv->offset + YDstOffset,
-	YDstPitch, YSrcPitch, lines, pixels);
+    LXCopyFromSys(pGeode, buf + YSrcOffset,
+	pPriv->vidmem->offset + YDstOffset, YDstPitch, YSrcPitch, lines,
+	pixels);
 
     /* Copy U + V at the same time */
 
-    LXCopyFromSys(pGeode, buf + USrcOffset, pPriv->offset + UDstOffset,
-	UVDstPitch, UVSrcPitch, lines, pixels >> 1);
+    LXCopyFromSys(pGeode, buf + USrcOffset,
+	pPriv->vidmem->offset + UDstOffset, UVDstPitch, UVSrcPitch, lines,
+	pixels >> 1);
 
-    videoScratch.dstOffset = pPriv->offset + YDstOffset;
+    videoScratch.dstOffset = pPriv->vidmem->offset + YDstOffset;
     videoScratch.dstPitch = YDstPitch;
     videoScratch.UVPitch = UVDstPitch;
-    videoScratch.UDstOffset = pPriv->offset + UDstOffset;
-    videoScratch.VDstOffset = pPriv->offset + VDstOffset;
+    videoScratch.UDstOffset = pPriv->vidmem->offset + UDstOffset;
+    videoScratch.VDstOffset = pPriv->vidmem->offset + VDstOffset;
 
     return TRUE;
 }
@@ -315,10 +302,8 @@ LXCopyPacked(ScrnInfoPtr pScrni, int id, unsigned char *buf,
 
     lines = ((dstPitch * height) + pGeode->Pitch - 1) / pGeode->Pitch;
 
-    pPriv->offset = LXAllocateVidMem(pScrni, &pPriv->area, height * dstPitch);
-
-    if (pPriv->offset == 0) {
-	ErrorF("Error while allocating an offscreen region.\n");
+    if (LXAllocateVidMem(pScrni, pPriv, lines) == FALSE) {
+	ErrorF("Error allocating an offscreen Packed region.\n");
 	return FALSE;
     }
 
@@ -335,7 +320,7 @@ LXCopyPacked(ScrnInfoPtr pScrni, int id, unsigned char *buf,
     srcOffset = (top * srcPitch) + left;
 
     /* Calculate the destination offset */
-    dstOffset = pPriv->offset + (top * dstPitch) + left;
+    dstOffset = pPriv->vidmem->offset + (top * dstPitch) + left;
 
     /* Make the copy happen */
 
@@ -481,6 +466,7 @@ LXPutImage(ScrnInfoPtr pScrni,
     GeodePortPrivRec *pPriv = (GeodePortPrivRec *) data;
     INT32 x1, x2, y1, y2;
     BoxRec dstBox;
+    Bool ret;
 
     if (pGeode->rotation != RR_Rotate_0)
 	return Success;
@@ -513,20 +499,16 @@ LXPutImage(ScrnInfoPtr pScrni,
     dstBox.y1 -= pScrni->frameY0;
     dstBox.y2 -= pScrni->frameY0;
 
-    switch (id) {
-    case FOURCC_YV12:
-    case FOURCC_I420:
-	LXCopyPlanar(pScrni, id, buf, x1, y1, x2, y2, width, height, data);
-	break;
-
-    case FOURCC_UYVY:
-    case FOURCC_YUY2:
-    case FOURCC_Y800:
-    case FOURCC_RGB565:
-	LXCopyPacked(pScrni, id, buf, x1, y1, x2, y2, width, height, data);
-	break;
-    }
+    if (id == FOURCC_YV12 || id == FOURCC_I420)
+	ret = LXCopyPlanar(pScrni, id, buf, x1, y1, x2, y2, width,
+			height, data);
+    else
+	ret = LXCopyPacked(pScrni, id, buf, x1, y1, x2, y2, width,
+			height, data);
 
+    if (ret == FALSE)
+	return BadAlloc;
+	
     if (!RegionsEqual(&pPriv->clip, clipBoxes) ||
 	(drawW != pPriv->pwidth || drawH != pPriv->pheight)) {
 	REGION_COPY(pScrni->pScreen, &pPriv->clip, clipBoxes);
@@ -542,7 +524,6 @@ LXPutImage(ScrnInfoPtr pScrni,
     }
 
     pPriv->videoStatus = CLIENT_VIDEO_ON;
-    pGeode->OverlayON = TRUE;
 
     return Success;
 }
@@ -604,7 +585,6 @@ static void
 LXStopVideo(ScrnInfoPtr pScrni, pointer data, Bool exit)
 {
     GeodePortPrivRec *pPriv = (GeodePortPrivRec *) data;
-    GeodeRec *pGeode = GEODEPTR(pScrni);
 
     if (pPriv->videoStatus == 0)
 	return;
@@ -622,15 +602,14 @@ LXStopVideo(ScrnInfoPtr pScrni, pointer data, Bool exit)
 	    WRITE_VID32(DF_VID_MISC, val | DF_GAMMA_BYPASS_BOTH);
 	}
 
-	if (pPriv->area) {
-	    exaOffscreenFree(pScrni->pScreen, pPriv->area);
-	    pPriv->area = NULL;
+	if (pPriv->vidmem) {
+	    exaOffscreenFree(pScrni->pScreen, pPriv->vidmem);
+	    pPriv->vidmem = NULL;
 	}
 
 	pPriv->videoStatus = 0;
 
 	/* Eh? */
-	pGeode->OverlayON = FALSE;
     } else if (pPriv->videoStatus & CLIENT_VIDEO_ON) {
 	pPriv->videoStatus |= OFF_TIMER;
 	pPriv->offTime = currentTime.milliseconds + OFF_DELAY;
@@ -684,9 +663,10 @@ LXVidBlockHandler(int i, pointer blockData, pointer pTimeout,
 	    }
 	} else {
 	    if (pPriv->freeTime < now) {
-		if (pPriv->area) {
-		    exaOffscreenFree(pScrni->pScreen, pPriv->area);
-		    pPriv->area = NULL;
+
+		if (pPriv->vidmem) {
+		    exaOffscreenFree(pScrni->pScreen, pPriv->vidmem);
+		    pPriv->vidmem = NULL;
 		}
 
 		pPriv->videoStatus = 0;
@@ -703,7 +683,7 @@ LXSetupImageVideo(ScreenPtr pScrn)
     XF86VideoAdaptorPtr adapt;
     GeodePortPrivRec *pPriv;
 
-    adapt = xcalloc(1, sizeof(XF86VideoAdaptorRec) +
+    adapt = calloc(1, sizeof(XF86VideoAdaptorRec) +
 	sizeof(GeodePortPrivRec) + sizeof(DevUnion));
 
     if (adapt == NULL) {
@@ -740,8 +720,9 @@ LXSetupImageVideo(ScreenPtr pScrn)
     /* Use the common function */
     adapt->QueryImageAttributes = GeodeQueryImageAttributes;
 
+    pPriv->vidmem = NULL;
     pPriv->filter = 0;
-    pPriv->colorKey = pGeode->videoKey;
+    pPriv->colorKey = 0;
     pPriv->colorKeyMode = 0;
     pPriv->videoStatus = 0;
     pPriv->pwidth = 0;
@@ -767,8 +748,7 @@ LXSetupImageVideo(ScreenPtr pScrn)
 
 struct OffscreenPrivRec
 {
-    void *area;
-    int offset;
+    ExaOffscreenArea *vidmem;
     Bool isOn;
 };
 
@@ -825,9 +805,8 @@ LXAllocateSurface(ScrnInfoPtr pScrni, int id, unsigned short w,
     unsigned short h, XF86SurfacePtr surface)
 {
     GeodeRec *pGeode = GEODEPTR(pScrni);
-    void *area = NULL;
     int pitch, lines;
-    unsigned offset;
+    ExaOffscreenArea *vidmem;
     struct OffscreenPrivRec *pPriv;
 
     if (w > 1024 || h > 1024)
@@ -839,9 +818,10 @@ LXAllocateSurface(ScrnInfoPtr pScrni, int id, unsigned short w,
     pitch = ((w << 1) + 15) & ~15;
     lines = ((pitch * h) + (pGeode->Pitch - 1)) / pGeode->Pitch;
 
-    offset = LXAllocateVidMem(pScrni, &area, lines);
+    vidmem = exaOffscreenAlloc(pScrni->pScreen, lines, 4, TRUE,
+		NULL, NULL);
 
-    if (offset == 0) {
+    if (vidmem == NULL) {
 	ErrorF("Error while allocating an offscreen region.\n");
 	return BadAlloc;
     }
@@ -849,36 +829,37 @@ LXAllocateSurface(ScrnInfoPtr pScrni, int id, unsigned short w,
     surface->width = w;
     surface->height = h;
 
-    surface->pitches = xalloc(sizeof(int));
+    surface->pitches = malloc(sizeof(int));
 
-    surface->offsets = xalloc(sizeof(int));
+    surface->offsets = malloc(sizeof(int));
 
-    pPriv = xalloc(sizeof(struct OffscreenPrivRec));
+    pPriv = malloc(sizeof(struct OffscreenPrivRec));
 
     if (pPriv && surface->pitches && surface->offsets) {
 
-	pPriv->area = area;
-	pPriv->offset = offset;
+	pPriv->vidmem = vidmem;
 
 	pPriv->isOn = FALSE;
 
 	surface->pScrn = pScrni;
 	surface->id = id;
 	surface->pitches[0] = pitch;
-	surface->offsets[0] = offset;
+	surface->offsets[0] = vidmem->offset;
 	surface->devPrivate.ptr = (pointer) pPriv;
 
 	return Success;
     }
 
     if (surface->offsets)
-	xfree(surface->offsets);
+	free(surface->offsets);
 
     if (surface->pitches)
-	xfree(surface->pitches);
+	free(surface->pitches);
 
-    if (area)
-	exaOffscreenFree(pScrni->pScreen, area);
+    if (vidmem) {
+	exaOffscreenFree(pScrni->pScreen, vidmem);
+	vidmem = NULL;
+    }
 
     return BadAlloc;
 }
@@ -903,14 +884,14 @@ LXFreeSurface(XF86SurfacePtr surface)
     if (pPriv->isOn)
 	LXStopSurface(surface);
 
-    if (pPriv->area) {
-	exaOffscreenFree(pScrni->pScreen, pPriv->area);
-	pPriv->area = NULL;
+    if (pPriv->vidmem) {
+	exaOffscreenFree(pScrni->pScreen, pPriv->vidmem);
+	pPriv->vidmem = NULL;
     }
 
-    xfree(surface->pitches);
-    xfree(surface->offsets);
-    xfree(surface->devPrivate.ptr);
+    free(surface->pitches);
+    free(surface->offsets);
+    free(surface->devPrivate.ptr);
 
     return Success;
 }
@@ -935,7 +916,7 @@ LXInitOffscreenImages(ScreenPtr pScrn)
     XF86OffscreenImagePtr offscreenImages;
 
     /* need to free this someplace */
-    if (!(offscreenImages = xalloc(sizeof(XF86OffscreenImageRec))))
+    if (!(offscreenImages = malloc(sizeof(XF86OffscreenImageRec))))
 	return;
 
     offscreenImages[0].image = &Images[0];
@@ -984,7 +965,7 @@ LXInitVideo(ScreenPtr pScrn)
 	adaptors = &newAdaptor;
     } else {
 	newAdaptors =
-	    xalloc((num_adaptors + 1) * sizeof(XF86VideoAdaptorPtr *));
+	    malloc((num_adaptors + 1) * sizeof(XF86VideoAdaptorPtr *));
 
 	if (newAdaptors) {
 	    memcpy(newAdaptors, adaptors, num_adaptors *
@@ -1000,5 +981,5 @@ LXInitVideo(ScreenPtr pScrn)
 	xf86XVScreenInit(pScrn, adaptors, num_adaptors);
 
     if (newAdaptors)
-	xfree(newAdaptors);
+	free(newAdaptors);
 }
